local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- LibMapData
local LibMapData = LibStub:GetLibrary("LibMapData-1.0")

-- LibTourist
local LibTourist = LibStub:GetLibrary("LibTourist-3.0")

-- LibNodeTypeInfo
local LibNTI = LibStub:GetLibrary("LibNodeTypeInfo-1.0")

-- the module name
local MODULE      = "Gatherer"
local DESCRIPTION = L["Module that uses Gatherer as data source."]

-- get Gatherer
local Gatherer = _G["Gatherer"]

-- local functions
local pairs = pairs

-- constants
local GATHERER_MINING    = "MINE"
local GATHERER_HERBALISM = "HERB"
local GATHERER_FISHING   = "FISH"
local GATHERER_LOGGING   = "TREE"

-- lookup tables
local mapProfession = {
	gathererToNTI = {
		[GATHERER_MINING]    = LibNTI.PROFESSION_MINING,
		[GATHERER_HERBALISM] = LibNTI.PROFESSION_HERBALISM,
		[GATHERER_FISHING]   = LibNTI.PROFESSION_FISHING,
		[GATHERER_LOGGING]   = LibNTI.PROFESSION_LOGGING,
	},
	ntiToGatherer = {
		[LibNTI.PROFESSION_MINING]    = GATHERER_MINING,
		[LibNTI.PROFESSION_HERBALISM] = GATHERER_HERBALISM,
		[LibNTI.PROFESSION_FISHING]   = GATHERER_FISHING,
		[LibNTI.PROFESSION_LOGGING]   = GATHERER_LOGGING,
	},
}

-- aux variables
local _

-- the module
local Inlet = NS:CreateInlet(MODULE, DESCRIPTION)

Inlet:SetReady(Gatherer and LibMapData and LibNTI and LibTourist and true or false)

-- variables
Inlet.nodeTypesGatherer = {}

-- register
NS:RegisterModule(Inlet)

function Inlet:GetNodesIterator(zoneId, profession)
	if not self:IsReady() then
		return
	end
	
	return self:IterateNodes(zoneId, mapProfession.ntiToGatherer[profession], profession)
end

do --Do-end block for iterator
	local emptyTbl = {}
	local tablestack = setmetatable({}, {__mode = 'k'})

	local function dbNodeIter(t, prestate)
		if not t then 
			return nil 
		end

		local iterator = t.iterator
		local tbl      = t.t
		
		if iterator then
			local nodeIndex, x, y = iterator(tbl, prestate)

			while nodeIndex do
				-- TODO: make it work for all nodes, atm only first hit is met -- which actually is enough
				for _, nodeId, count in Gatherer.Storage.GetNodeGatherNames(t.zoneId, t.profession, nodeIndex) do
					if count > 0 and Inlet.nodeTypesGatherer[nodeId] then
						local encLoc = LibMapData:EncodeLoc(x, y, 1)
						-- state, encLoc, nodeName
						return nodeIndex, encLoc, LibNTI:GetName(t.ntiProfession, nodeId) -- Gatherer.Util.GetNodeName(nodeId)
					end
				end
				
				nodeIndex, x, y = iterator(tbl, nodeIndex)
			end	
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function Inlet:IterateNodes(zoneId, profession, ntiProfession)
		local tbl = next(tablestack) or {}

		tablestack[tbl] = nil
		
		local iterator, t, state = Gatherer.Storage.ZoneGatherNodes(zoneId, profession)
		
		tbl.iterator      = iterator
		tbl.t             = t
		tbl.ntiProfession = ntiProfession
		tbl.profession    = profession
		tbl.zoneId        = zoneId
		
		return dbNodeIter, tbl, state
	end
end

-- implement update function which is called after node type selection changed in magneto
function Inlet:UpdateNodeSelection(event)
	if not self:IsReady() then
		return
	end
	
	NS:ClearTable(self.nodeTypesGatherer)
	
	for nodeId, _ in Addon:IterateNodeTypeSelection() do
		-- gatherer uses the same node type ids as LibNTI
		self.nodeTypesGatherer[nodeId] = true
	end
end

-- test
function Inlet:Debug(msg)
	Addon:Debug("(Inlet:" .. MODULE .. ") " .. msg)
end
